#ifndef __ASRDEV_H__
#define __ASRDEV_H__

#include <linux/string.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/efi.h>
#include <linux/utsname.h>
#include <linux/version.h>
#include <linux/io.h>

/* 
 * <linux/uaccess.h> was added in kernel version 2.6.18, and should be
 * included in preference to <asm/uaccess.h>.  In particular, copy_to_user()
 * and copy_from_user() were moved to <linux/uaccess.h> in kernel version
 * 4.12.
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)
#include <linux/uaccess.h>
#else
#include <asm/uaccess.h>
#endif

/*
 * Kernel 5.0 removed VERIFY_READ and VERIFY_WRITE and removed the first
 * parameter of access_ok() which was set to VERIFY_READ or VERIFY_WRITE.
 * That has been redundant since kernel 2.5.70, and even then it was only
 * checked for kernels that support old 386 processors.
 *
 * Get rid of the first parameter and always pass VERIFY_WRITE for kernels
 * prior to 5.0.  This will fail for old 386 processors on pre-2.5.70
 * kernels if the memory region is not in fact writeable.
 */
#ifdef VERIFY_WRITE
/* Pre 5.0 kernel. */
static inline int _kcompat_access_ok(unsigned long addr, size_t size)
{
    /* Always use VERIFY_WRITE.  Most architectures ignore it. */
    return access_ok(VERIFY_WRITE, addr, size);
}
/* Redefine access_ok() to remove first parameter. */
#undef access_ok
#define access_ok(addr, size) _kcompat_access_ok((unsigned long)(addr), (size))
#endif

#include "asrdev_ioctl.h"

#undef PDEBUG
#ifdef USE_DEBUG
    #define PDEBUG(fmt, args...) printk(KERN_DEBUG "ASRDEV: " fmt, ## args)
#else
    #define PDEBUG(fmt, args...)
#endif // ASRDEV_DEBUG

#define DRIVER_NAME             "asrdev"
#define DEVICE_NAME             "asrdev"
#define VER                     "v1.0.0"

static int asrdev_major         = 0;
static int asrdev_minor         = 0;
static int asrdev_num_devs      = 1;
int kernel_major = 0;
int kernel_minor = 0;
int kernel_build = 0;
static struct class* charmodule_class;


module_param(asrdev_major, int, S_IRUGO);
module_param(asrdev_minor, int, S_IRUGO);
module_param(asrdev_num_devs, int, S_IRUGO);

struct asrdev_data
{
    struct cdev cdev;
    struct mutex mutex;
};
static struct asrdev_data* asrdev_devices;

static int __init asrdev_module_init(void);
static void __exit asrdev_module_exit(void);
module_init(asrdev_module_init);
module_exit(asrdev_module_exit);

static void asrdev_setup_cdev(struct asrdev_data* dev, int index);
static void asrdev_module_clean(void);

int asrdev_module_open(struct inode* inode, struct file* file);
int asrdev_module_release(struct inode* inode, struct file* file);
long asrdev_module_ioctl(struct file* file, unsigned int cmd, unsigned long arg);

static int SmiFunction(PSmiPara volatile SysBuf);

static struct file_operations asrdev_fops = 
{
    .owner = THIS_MODULE,
    .unlocked_ioctl = asrdev_module_ioctl,
    .compat_ioctl = asrdev_module_ioctl,
    .open = asrdev_module_open,
    .release = asrdev_module_release
};

//MODULE_LICENSE("Proprietary");
MODULE_LICENSE("Dual BSD/GPL");
MODULE_AUTHOR("www.asrock.com.tw");
MODULE_VERSION("1.0.0");
MODULE_DESCRIPTION("ASRock system device driver");

#endif // __ASRDEV_H__
